@@ -1,7 +1,7 @@
 """PyTorch implementations of the datafactory module contents."""
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Any, List, Mapping, Optional, Union
+from typing import TYPE_CHECKING, Any, List, Mapping, Optional, Union, cast
 
 from bitfount.backends.pytorch.data.dataloaders import (
     PyTorchBitfountDataLoader,
@@ -13,8 +13,8 @@ from bitfount.backends.pytorch.data.datasets import (
     _PyTorchIterableDataset,
 )
 from bitfount.data.datafactory import _DataFactory
-from bitfount.data.datasources.base_source import BaseSource
-from bitfount.data.datasources.database_source import DatabaseSource
+from bitfount.data.datasources.base_source import BaseSource, IterableSource
+from bitfount.data.datasources.views import DataView
 
 if TYPE_CHECKING:
     from bitfount.data.datasets import _BaseBitfountDataset
@@ -47,7 +47,7 @@ class _PyTorchDataFactory(_DataFactory):
 
     def create_dataset(
         self,
-        datasource: BaseSource,
+        datasource: Union[BaseSource, DataView],
         data_splitter: Optional[DatasetSplitter],
         data_split: DataSplit,
         schema: TableSchema,
@@ -59,12 +59,12 @@ class _PyTorchDataFactory(_DataFactory):
         **kwargs: Any,
     ) -> Union[_PyTorchDataset, _PyTorchIterableDataset]:
         """See base class."""
-        if datasource.multi_table and isinstance(datasource, DatabaseSource):
+        if datasource.iterable:
             return _PyTorchIterableDataset(
                 schema=schema,
                 selected_cols_semantic_types=selected_cols_semantic_types,
                 data_splitter=data_splitter,
-                datasource=datasource,
+                datasource=cast(IterableSource, datasource),
                 target=target,
                 selected_cols=selected_cols,
                 batch_transforms=batch_transforms,
