@@ -81,6 +81,7 @@ class _BaseBitfountDataset(ABC):
         multihead_col: Optional[str] = None,
         ignore_classes_col: Optional[str] = None,
         auto_convert_grayscale_images: bool = True,
+        ignore_support_cols: bool = False,
     ) -> None:
         super().__init__()
         self.datasource = datasource
@@ -95,6 +96,7 @@ class _BaseBitfountDataset(ABC):
         self.multihead_col = multihead_col
         self.ignore_classes_col = ignore_classes_col
         self.auto_convert_grayscale_images = auto_convert_grayscale_images
+        self.ignore_support_cols = ignore_support_cols
 
         # Empty placeholder arrays for images - will be populated later if necessary
         self.image = np.array([])
@@ -223,7 +225,9 @@ class _BaseBitfountDataset(ABC):
         elif what_to_load == "target":
             img_features = np.array([self.y_var[idx]])
         imgs: Tuple[np.ndarray, ...] = tuple(
-            io.imread(image, plugin="pil") for image in img_features
+            io.imread(image, plugin="pil")
+            for image in img_features
+            if image is not pd.NA and image is not np.nan
         )
         if self.auto_convert_grayscale_images:
             imgs = tuple(
@@ -262,17 +266,17 @@ class _BaseBitfountDataset(ABC):
             category = data.loc[:, [self.multihead_col]].values
             category = category.reshape(len(category), 1)
             self.support_cols = cast(
-                np.ndarray, np.concatenate((weights, ignore_classes, category), axis=1)
+                np.ndarray, np.concatenate([weights, ignore_classes, category], axis=1)
             )
         else:
             self.support_cols = cast(
-                np.ndarray, np.concatenate((weights, ignore_classes), axis=1)
+                np.ndarray, np.concatenate([weights, ignore_classes], axis=1)
             )
 
     def _set_image_values(self, data: pd.DataFrame) -> None:
         """Sets `self.image`."""
         for i, col in enumerate(self.image_columns):
-            x_img = np.expand_dims(data.loc[:, col].values, axis=1)
+            x_img = np.expand_dims(cast(np.ndarray, data.loc[:, col].values), axis=1)
             # If there are multiple images, we start concatentating `self.image`
             # with each next image
             self.image = (
@@ -282,10 +286,16 @@ class _BaseBitfountDataset(ABC):
     def _set_tabular_values(self, data: pd.DataFrame) -> None:
         """Sets `self.tabular`."""
         x1_var = data.loc[:, self.embedded_col_names].values.astype(np.int64)
-        x2_var = data.loc[
-            :, self.selected_cols_semantic_types.get("continuous", [])
-        ].values.astype(np.float32)
-        self.tabular = np.concatenate((x1_var, x2_var), axis=1)
+        # Fill NaTypes to make sure it does not error (due to files loading iteratively)
+        # and having missing columns when loading small batches. For the
+        # non-iterable datasets, this just replaces all `nan` by 0,
+        # similar to the `CleanDataTransformation`.
+        x2_var = (
+            data.loc[:, self.selected_cols_semantic_types.get("continuous", [])]
+            .fillna(value=0.0)
+            .values.astype(np.float32)
+        )
+        self.tabular = np.concatenate([x1_var, x2_var], axis=1)
 
     def _set_target_values(
         self, target: Optional[Union[pd.DataFrame, pd.Series]]
@@ -336,25 +346,30 @@ class _BaseBitfountDataset(ABC):
             tab = self.tabular[idx]
             sup = self.support_cols[idx]
             image = self._load_images(idx)
+            if self.ignore_support_cols:
+                return (tab, image), target
             return (tab, image, sup), target
 
         # If the Dataset contains only tabular data
         elif self.tabular.size:
             tab = self.tabular[idx]
             sup = self.support_cols[idx]
+            if self.ignore_support_cols:
+                return tab, target
             return (tab, sup), target
 
         # If the Dataset contains only image data
         else:
             sup = self.support_cols[idx]
             image = self._load_images(idx)
+            if self.ignore_support_cols:
+                return image, target
             return (image, sup), target
 
     def _reformat_data(self, data: pd.DataFrame) -> None:
         """Reformats the data to be compatible with the Dataset class."""
         self._apply_schema(data)
         X, Y = self._get_xy(self.data)
-
         if self.image_columns:
             self._set_image_values(X)
         self._set_tabular_values(X)
@@ -415,6 +430,7 @@ class _BitfountDataset(_BaseBitfountDataset):
         multihead_col: Optional[str] = None,
         ignore_classes_col: Optional[str] = None,
         auto_convert_grayscale_images: bool = True,
+        ignore_support_cols: bool = False,
     ) -> None:
         super().__init__(
             datasource=datasource,
@@ -429,6 +445,7 @@ class _BitfountDataset(_BaseBitfountDataset):
             multihead_col=multihead_col,
             ignore_classes_col=ignore_classes_col,
             auto_convert_grayscale_images=auto_convert_grayscale_images,
+            ignore_support_cols=ignore_support_cols,
         )
 
         data = self.get_dataset_split(
@@ -479,7 +496,7 @@ class _BitfountDataset(_BaseBitfountDataset):
         """
         if not self.datasource._data_is_split:
             data_splitter = self._resolve_data_splitter(data_splitter)
-            assert self.datasource.data is not None  # nosec[assert_used]
+            assert self.datasource.data is not None  # nosec assert_used
 
             (
                 self.datasource._train_idxs,
@@ -528,6 +545,9 @@ class _IterableBitfountDataset(_BaseBitfountDataset):
         """
         data_splitter = self._resolve_data_splitter(data_splitter)
 
+        if isinstance(self.datasource, FileSystemIterableSource):
+            return len(data_splitter.get_filenames(self.datasource, split))
+
         # If PercentageSplitter is used regardless of the data loader.
         if isinstance(data_splitter, PercentageSplitter):
             len_datasource = len(self.datasource)
@@ -546,7 +566,11 @@ class _IterableBitfountDataset(_BaseBitfountDataset):
             with self.datasource.con.connect() as con:
                 # Ignoring the security warning because the sql query is trusted and
                 # will be executed regardless.
-                result = con.execute(text(f"SELECT COUNT(*) FROM ({query}) q"))  # nosec
+                result = con.execute(
+                    text(
+                        f"SELECT COUNT(*) FROM ({query}) q"  # nosec hardcoded_sql_expressions # noqa: B950
+                    )
+                )
                 return cast(int, result.scalar_one())
 
         # `load_data` should be called to avoid this error being raised
@@ -582,7 +606,6 @@ class _IterableBitfountDataset(_BaseBitfountDataset):
         """
         kwargs: Dict[str, Any] = {}
         data_splitter = self._resolve_data_splitter(data_splitter)
-
         if isinstance(self.datasource, DatabaseSource):
             if not self.datasource.query:
                 raise ValueError("No query or table name specified.")
