@@ -28,8 +28,8 @@ from pandas.core.dtypes.common import is_numeric_dtype
 import yaml
 
 import bitfount
-from bitfount.data import BaseSource
 from bitfount.data.datasources.base_source import (
+    BaseSource,
     FileSystemIterableSource,
     MultiTableSource,
 )
@@ -42,6 +42,7 @@ from bitfount.data.types import (
     TextRecord,
     _CamelCaseSchema,
     _FeatureDict,
+    _ForceStypeValue,
     _SemanticTypeRecord,
     _SemanticTypeValue,
 )
@@ -278,7 +279,9 @@ class TableSchema(_TableSchemaMarshmallowMixIn):
         self,
         datasource: BaseSource,
         ignore_cols: List[str],
-        force_stype: MutableMapping[_SemanticTypeValue, List[str]],
+        force_stype: MutableMapping[
+            Union[_ForceStypeValue, _SemanticTypeValue], List[str]
+        ],
         descriptions: Mapping[str, str],
     ) -> None:
         """Add given dataframe to the schema.
@@ -287,13 +290,34 @@ class TableSchema(_TableSchemaMarshmallowMixIn):
         the semantic type of the feature.
         """
         table_dtypes = datasource.get_dtypes(table_name=self.name)
-        for item in force_stype.values():
+        # Get all image columns from 'image_prefix' force_stype.
+        if "image_prefix" in force_stype:
+            # if image_prefix is provided with a list of strings,
+            # we iterate through all the datasource columns to
+            # see which contain the image prefix.
+            img_cols = [
+                col
+                for col in datasource.data.columns
+                if any(col.startswith(stype) for stype in force_stype["image_prefix"])
+            ]
+            if "image" in force_stype:
+                force_stype["image"] = _add_this_to_list(img_cols, force_stype["image"])
+            else:
+                force_stype["image"] = img_cols
+            force_stype.pop("image_prefix")
+        # After we extract the image features based on image prefix,
+        # it's safe to cast as we remove `image_prefix` from force_stype
+        forced_stype: MutableMapping[_SemanticTypeValue, List[str]] = cast(
+            MutableMapping[_SemanticTypeValue, List[str]], force_stype
+        )
+
+        for item in forced_stype.values():
             ignore_cols = _add_this_to_list(item, ignore_cols)
         inferred_semantic_types = self._dtype_based_stype_split(
             table_dtypes, ignore_cols
         )
         semantic_types = self._combine_existing_stypes_with_forced_stypes(
-            inferred_semantic_types, force_stype, table_dtypes
+            inferred_semantic_types, forced_stype, table_dtypes
         )
         if SemanticType.CATEGORICAL in semantic_types:
             values = datasource.get_values(
@@ -305,7 +329,14 @@ class TableSchema(_TableSchemaMarshmallowMixIn):
             # This ensures they are added in deterministic order.
             features.sort()
             for feature_name in features:
-                dtype = table_dtypes[feature_name]
+                if feature_name not in table_dtypes:
+                    logger.warning(
+                        f"Column {feature_name} does not have a data type. "
+                        "Skipping it as it is probably not in the source data."
+                    )
+                    continue
+                else:
+                    dtype = table_dtypes[feature_name]
                 description = descriptions.get(feature_name)
                 if stype == SemanticType.TEXT:
                     if feature_name not in self.get_feature_names():
@@ -352,7 +383,6 @@ class TableSchema(_TableSchemaMarshmallowMixIn):
                             dtype=dtype,
                             description=descriptions.get(feature_name),
                         )
-
                     # If the datasource is a FileSystemIterableSource and `fast_load` is
                     # True, we won't iterate over the images to get their shapes and
                     # formats, etc.
@@ -365,10 +395,11 @@ class TableSchema(_TableSchemaMarshmallowMixIn):
                     for img in datasource.get_column(
                         feature_name, table_name=self.name
                     ):
-                        im = Image.open(img)
-                        record.dimensions[im.size] += 1
-                        record.modes[im.mode] += 1
-                        record.formats[im.format] += 1
+                        if img is not None and img is not np.nan:
+                            im = Image.open(img)
+                            record.dimensions[im.size] += 1
+                            record.modes[im.mode] += 1
+                            record.formats[im.format] += 1
 
     def _expand_dataframe(self, dataframe: pd.DataFrame) -> pd.DataFrame:
         """Expands dataframe to include missing columns specified in the schema.
@@ -511,7 +542,7 @@ class TableSchema(_TableSchemaMarshmallowMixIn):
                 except ValueError as err:
                     raise ValueError(
                         f"Could not encode column {feature_name}: {str(err)}"
-                    )
+                    ) from err
         else:
             logger.info("No encoding to be done as there are no categorical features.")
 
@@ -633,7 +664,9 @@ class TableSchema(_TableSchemaMarshmallowMixIn):
         self,
         datasource: BaseSource,
         ignore_cols: Optional[Sequence[str]] = None,
-        force_stype: Optional[MutableMapping[_SemanticTypeValue, List[str]]] = None,
+        force_stype: Optional[
+            MutableMapping[Union[_ForceStypeValue, _SemanticTypeValue], List[str]]
+        ] = None,
         descriptions: Optional[Mapping[str, str]] = None,
     ) -> None:
         """Adds datasource features to schema.
@@ -910,7 +943,10 @@ class BitfountSchema(_BitfountSchemaMarshmallowMixIn):
         column_descriptions: Optional[Mapping[str, Mapping[str, str]]] = None,
         ignore_cols: Optional[Mapping[str, Sequence[str]]] = None,
         force_stypes: Optional[
-            Mapping[str, MutableMapping[_SemanticTypeValue, List[str]]]
+            Mapping[
+                str,
+                MutableMapping[Union[_SemanticTypeValue, _ForceStypeValue], List[str]],
+            ]
         ] = None,
     ) -> None:
         """Adds the tables from a `BaseSource` to the schema.
@@ -943,8 +979,8 @@ class BitfountSchema(_BitfountSchemaMarshmallowMixIn):
             )
         if not self._frozen:
             if datasource.multi_table:
-                assert isinstance(datasource, MultiTableSource)  # nosec[assert_used]
-                assert datasource.table_names is not None  # nosec[assert_used]
+                assert isinstance(datasource, MultiTableSource)  # nosec assert_used
+                assert datasource.table_names is not None  # nosec assert_used
 
                 table_names = datasource.table_names
                 if table_name:
