@@ -19,6 +19,7 @@ from typing import (
     Optional,
     Sequence,
     Tuple,
+    Type,
     TypedDict,
     TypeVar,
     Union,
@@ -57,12 +58,25 @@ _ImagesData = _SingleOrMulti[np.ndarray]
 _TabularData = np.ndarray
 _SupportData = np.ndarray
 _Y_VAR = np.ndarray
-_TabularEntry = Tuple[Tuple[_TabularData, _SupportData], _Y_VAR]
-_ImageEntry = Tuple[Tuple[_ImagesData, _SupportData], _Y_VAR]
-_ImageAndTabularEntry = Tuple[Tuple[_TabularData, _ImagesData, _SupportData], _Y_VAR]
-_Segmentation_ImageEntry = Tuple[Tuple[_ImagesData, _SupportData], _ImagesData]
-_Segmentation_ImageAndTabEntry = Tuple[
-    Tuple[_TabularData, _ImagesData, _SupportData], _ImagesData
+_TabularEntry = Union[
+    Tuple[Tuple[_TabularData, _SupportData], _Y_VAR],
+    Tuple[_TabularData, _Y_VAR],
+]
+_ImageEntry = Union[
+    Tuple[Tuple[_ImagesData, _SupportData], _Y_VAR],
+    Tuple[_ImagesData, _Y_VAR],
+]
+_ImageAndTabularEntry = Union[
+    Tuple[Tuple[_TabularData, _ImagesData, _SupportData], _Y_VAR],
+    Tuple[Tuple[_TabularData, _ImagesData], _Y_VAR],
+]
+_Segmentation_ImageEntry = Union[
+    Tuple[Tuple[_ImagesData, _SupportData], _ImagesData],
+    Tuple[_ImagesData, _ImagesData],
+]
+_Segmentation_ImageAndTabEntry = Union[
+    Tuple[Tuple[_TabularData, _ImagesData, _SupportData], _ImagesData],
+    Tuple[Tuple[_TabularData, _ImagesData], _ImagesData],
 ]
 _DataEntry = Union[
     _TabularEntry,
@@ -75,6 +89,7 @@ _DataBatch = List[_DataEntry]
 
 # Type alias for the values in the `SemanticType` enum
 _SemanticTypeValue = Literal["categorical", "continuous", "image", "text"]
+_ForceStypeValue = Literal["categorical", "continuous", "image", "text", "image_prefix"]
 S = TypeVar("S", bound="BaseSource")
 _Column = Union[np.ndarray, pd.Series]
 _GetColumnCallable = Callable[
@@ -240,13 +255,38 @@ class _SemanticTypeRecord:
     def __init__(
         self,
         feature_name: str,
-        dtype: Union[Dtype, np.dtype],
+        dtype: Union[Dtype, np.dtype, Type[np.generic]],
         description: Optional[str] = None,
     ) -> None:
         self.feature_name = feature_name
-        self.dtype = dtype
+        self.dtype: Union[Dtype, np.dtype, Type[np.generic]] = dtype
         self.description = description
 
+    @property
+    def dtype_name(self) -> str:
+        """The string name of the dtype.
+
+        Returns "unknown" if unable to resolve.
+        """
+        # self.dtype can be any of:
+        #   - ExtensionDtype
+        #   - str instance
+        #   - np.dtype
+        #   - explicit types: str, complex, bool, object
+        #   - np scalar type (e.g. np.uint8)
+
+        # Handles ExtensionDtype and np.dtype
+        if hasattr(self.dtype, "name"):
+            return self.dtype.name
+        # Handles explicit types and np scalar types
+        elif isinstance(self.dtype, type):
+            return self.dtype.__name__
+        # Handles str instance
+        elif isinstance(self.dtype, str):
+            return self.dtype
+        else:
+            return "unknown"
+
     @property
     @abstractmethod
     def semantic_type(self) -> SemanticType:
@@ -289,11 +329,11 @@ class _SemanticTypeRecord:
             """
             try:
                 data["dtype"] = pandas_dtype(data["dtype"])
-            except TypeError:
+            except TypeError as e:
                 raise ValidationError(
                     f"Continuous record `dtype` expected a valid np.dtype or a "
                     f"pandas dtype but received: `{data['dtype']}`."
-                )
+                ) from e
             return data
 
 
@@ -562,6 +602,18 @@ class DataPathModifiers(TypedDict):
     prefix: NotRequired[str]
 
 
+class DatasourceType(Enum):
+    """Bitfount Datasource types."""
+
+    CSVSource = "bitfount.CSVSource"
+    DatabaseSource = "bitfount.DatabaseSource"
+    DataFrameSource = "bitfount.DataFrameSource"
+    DICOMSource = "bitfount.DICOMSource"
+    ExcelSource = "bitfount.ExcelSource"
+    DropColsDataview = "bitfount.DropColsDataview"
+    SQLDataView = "bitfount.SQLDataView"
+
+
 SchemaOverrideMapping = Mapping[
     _SemanticTypeValue, List[Union[str, Mapping[str, Mapping[str, int]]]]
 ]
