@@ -19,7 +19,7 @@ from typing import (
 
 from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey, RSAPublicKey
 
-from bitfount.config import BITFOUNT_STORAGE_PATH
+from bitfount.config import BITFOUNT_DEFAULT_BATCHED_EXECUTION, BITFOUNT_STORAGE_PATH
 from bitfount.federated.aggregators.secure import SecureAggregator
 from bitfount.federated.authorisation_checkers import (
     _IDENTITY_VERIFICATION_METHODS_MAP,
@@ -30,6 +30,7 @@ from bitfount.federated.authorisation_checkers import (
 from bitfount.federated.encryption import _RSAEncryption
 from bitfount.federated.exceptions import PodResponseError
 from bitfount.federated.helper import (
+    TaskContext,
     _check_and_update_pod_ids,
     _create_message_service,
     _get_idp_url,
@@ -276,6 +277,7 @@ class _Modeller:
         pod_identifiers: Iterable[str],
         project_id: Optional[str],
         run_on_new_data_only: bool = False,
+        batched_execution: Optional[bool] = None,
     ) -> _ModellerMailbox:
         """Sends task requests to pods.
 
@@ -284,15 +286,20 @@ class _Modeller:
             project_id: Project Id the task belongs to.
             run_on_new_data_only: Whether to run the task on new datapoints only.
                 Defaults to False.
+            batched_execution: Whether to run the task in batched mode. Defaults to
+                False.
 
         Returns:
             The created ModellerMailbox for the task.
         """
+        if batched_execution is None:
+            batched_execution = BITFOUNT_DEFAULT_BATCHED_EXECUTION
         # Load pod public keys
         loaded_pod_public_keys: Dict[str, RSAPublicKey] = _get_pod_public_keys(
             pod_identifiers=pod_identifiers,
             hub=self._hub,
             pod_public_key_paths=self._pod_public_key_paths,
+            project_id=project_id,
         )
 
         task_request_msg_gen = self._get_task_request_msg_gen()
@@ -307,6 +314,7 @@ class _Modeller:
             message_service=self._message_service,
             project_id=project_id,
             run_on_new_data_only=run_on_new_data_only,
+            batched_execution=batched_execution,
         )
 
         return modeller_mailbox
@@ -321,7 +329,7 @@ class _Modeller:
             authorization_checker_cls = cast(
                 Type[_SignatureBasedAuthorisation], authorization_checker_cls
             )
-            assert isinstance(self._private_key, RSAPrivateKey)  # nosec[assert_used]
+            assert isinstance(self._private_key, RSAPrivateKey)  # nosec assert_used
             return authorization_checker_cls.create_task_request_message_generator(
                 self._private_key
             )
@@ -334,6 +342,7 @@ class _Modeller:
         pod_identifiers: Iterable[str],
         require_all_pods: bool = False,
         response_handler: Optional[_ResponseHandler] = None,
+        batched_execution: Optional[bool] = None,
     ) -> Union[Literal[False], Optional[Any]]:
         """Waits for pod responses and handles any who don't respond in time.
 
@@ -342,7 +351,8 @@ class _Modeller:
         # If additional response handling is specified, we call the handler
         if response_handler:
             await response_handler.handle(modeller_mailbox)
-
+        if batched_execution is None:
+            batched_execution = BITFOUNT_DEFAULT_BATCHED_EXECUTION
         # Process job accept/reject
         await modeller_mailbox.process_task_request_responses()
 
@@ -374,18 +384,29 @@ class _Modeller:
                     unaccepted_worker_msg + "Continuing task without these pods ..."
                 )
 
-        await modeller_mailbox.send_task_start_message()
-
-        return await self._run_modeller_protocol(modeller_mailbox)
+        return await self._run_modeller_protocol(modeller_mailbox, batched_execution)
 
     async def _run_modeller_protocol(
         self,
         modeller_mailbox: _ModellerMailbox,
+        batched_execution: Optional[bool] = None,
     ) -> Optional[Any]:
         """Runs the modeller-side of the protocol."""
-        protocol = self.protocol.modeller(mailbox=modeller_mailbox)
-        # Run the task
-        result = await protocol.run()
+        if batched_execution is None:
+            batched_execution = BITFOUNT_DEFAULT_BATCHED_EXECUTION
+        modeller_protocol = self.protocol.modeller(mailbox=modeller_mailbox)
+
+        # Initialise the modeller side of the protocol
+        modeller_protocol.initialise(
+            task_id=modeller_mailbox.task_id,
+        )
+
+        # Run the modeller side of the protocol
+        result = await modeller_protocol.run(
+            batched_execution=batched_execution, context=TaskContext.MODELLER
+        )
+        # Send task complete message
+        await modeller_mailbox.send_task_complete_message()
         return result
 
     async def run_async(
@@ -394,6 +415,7 @@ class _Modeller:
         require_all_pods: bool,
         project_id: Optional[str] = None,
         run_on_new_data_only: bool = False,
+        batched_execution: Optional[bool] = None,
     ) -> Union[Literal[False], Optional[Any]]:
         """Runs the modeller's task with a set of pods.
 
@@ -406,6 +428,8 @@ class _Modeller:
             project_id: Project Id the task belongs to. Defaults to None.
             run_on_new_data_only: Whether to run the task on new datapoints only.
                 Defaults to False.
+            batched_execution: Whether to run the task in batched mode. Defaults to
+                False.
 
         Returns:
             Whatever the protocol's return value is.
@@ -414,13 +438,15 @@ class _Modeller:
             PodResponseError: If require_all_pods is True and at least one
                 pod_identifier rejects or fails to respond to the task request.
         """
+        if batched_execution is None:
+            batched_execution = BITFOUNT_DEFAULT_BATCHED_EXECUTION
         try:
             # We do this early to give any handler components time to spin up
             response_handler = self._get_response_handler()
 
             logger.info("Sending task requests...")
             modeller_mailbox: _ModellerMailbox = await self._send_task_requests(
-                pod_identifiers, project_id, run_on_new_data_only
+                pod_identifiers, project_id, run_on_new_data_only, batched_execution
             )
 
             with task_monitor_context(
@@ -436,12 +462,11 @@ class _Modeller:
                         pod_identifiers=pod_identifiers,
                         require_all_pods=require_all_pods,
                         response_handler=response_handler,
+                        batched_execution=batched_execution,
                     ),
                     modeller_mailbox,
                 )
                 modeller_mailbox.delete_all_handlers(_BitfountMessageType.LOG_MESSAGE)
-
-                logger.info("Task complete")
                 return run_result
         except PodDoesNotExistError as e:
             logger.exception(e)
@@ -532,6 +557,7 @@ class _Modeller:
         model_out: Optional[Union[Path, str]] = None,
         project_id: Optional[str] = None,
         run_on_new_data_only: bool = False,
+        batched_execution: bool = False,
     ) -> Union[Literal[False], Optional[Any]]:
         """Runs the modeller's task with a set of pods.
 
@@ -545,6 +571,8 @@ class _Modeller:
             project_id: Project_id the task belongs to.
             run_on_new_data_only: Whether to run the task on new datapoints only.
                 Defaults to False.
+            batched_execution: Whether to run the task in batched mode. Defaults to
+                False.
 
         Returns:
             Whatever the protocol's return value is.
@@ -562,10 +590,10 @@ class _Modeller:
         # the clipping at the 'SecureShare' level.
         if (
             hasattr(self.protocol, "aggregator")
-            and isinstance(self.protocol.aggregator, SecureAggregator)  # type: ignore[attr-defined] # Reason: see below # noqa: B950
+            and isinstance(self.protocol.aggregator, SecureAggregator)
         ) and (
             hasattr(self.protocol, "steps_between_parameter_updates")
-            and self.protocol.steps_between_parameter_updates != 1  # type: ignore[attr-defined] # Reason: see below # noqa: B950
+            and self.protocol.steps_between_parameter_updates != 1
         ):
             if any(hasattr(algo, "model") for algo in self.protocol.algorithms):
                 logger.warning(
@@ -577,7 +605,7 @@ class _Modeller:
             # they might have to implement clipping in their custom model.
             if any(
                 hasattr(algo, "model")
-                and isinstance(algo.model, BitfountModelReference)  # type: ignore[attr-defined] # Reason: see below # noqa: B950
+                and isinstance(algo.model, BitfountModelReference)
                 for algo in self.protocol.algorithms
             ):
                 logger.warning(
@@ -585,18 +613,21 @@ class _Modeller:
                     "We recommend clipping the model parameters."
                 )
 
-            # Mypy: hasattr check on the protocol aggregator, model and steps.
             # Pass the `SecureShare` parameters to the model for clipping
             for algo in self.protocol.algorithms:
                 if hasattr(algo, "model"):
-                    algo.model.param_clipping = {  # type: ignore[attr-defined] # Reason: see above # noqa: B950
-                        "prime_q": self.protocol.aggregator._secure_share.prime_q,  # type: ignore[attr-defined] # Reason: see above # noqa: B950
-                        "precision": self.protocol.aggregator._secure_share.precision,  # type: ignore[attr-defined] # Reason: see above # noqa: B950
+                    algo.model.param_clipping = {
+                        "prime_q": self.protocol.aggregator._secure_share.prime_q,
+                        "precision": self.protocol.aggregator._secure_share.precision,
                         "num_workers": len(pod_identifiers),
                     }
         result = asyncio.run(
             self.run_async(
-                pod_identifiers, require_all_pods, project_id, run_on_new_data_only
+                pod_identifiers,
+                require_all_pods,
+                project_id,
+                run_on_new_data_only,
+                batched_execution,
             )
         )
         if model_out and result is not False:
