@@ -21,7 +21,6 @@ from typing import (
 
 from marshmallow import fields
 
-from bitfount.data.datasources.base_source import BaseSource
 from bitfount.federated.aggregators.base import (
     _AggregatorWorkerFactory,
     _BaseAggregator,
@@ -38,10 +37,10 @@ from bitfount.federated.early_stopping import FederatedEarlyStopping
 from bitfount.federated.helper import _create_aggregator
 from bitfount.federated.logging import _get_federated_logger
 from bitfount.federated.model_reference import BitfountModelReference
-from bitfount.federated.pod_vitals import _PodVitals
-from bitfount.federated.privacy.differential import DPPodConfig
 from bitfount.federated.protocols.base import (
     BaseCompatibleAlgoFactory,
+    BaseCompatibleModellerAlgorithm,
+    BaseCompatibleWorkerAlgorithm,
     BaseModellerProtocol,
     BaseProtocolFactory,
     BaseWorkerProtocol,
@@ -71,8 +70,10 @@ from bitfount.types import (
 from bitfount.utils import delegates
 
 if TYPE_CHECKING:
+    from bitfount.federated.pod_vitals import _PodVitals
     from bitfount.types import _DistributedModelTypeOrReference
 
+
 logger = _get_federated_logger(__name__)
 
 
@@ -95,21 +96,13 @@ class _FederatedAveragingCompatibleAlgo(Protocol[T_DTYPE]):
 
 
 @runtime_checkable
-class _FederatedAveragingCompatibleWorker(_FederatedAveragingCompatibleAlgo, Protocol):
+class _FederatedAveragingCompatibleWorker(
+    _FederatedAveragingCompatibleAlgo, BaseCompatibleWorkerAlgorithm, Protocol
+):
     """Defines worker-side algorithm compatibility."""
 
-    def initialise(
-        self,
-        datasource: BaseSource,
-        pod_dp: Optional[DPPodConfig] = None,
-        **kwargs: Any,
-    ) -> None:
-        """Initialises the worker-side algorithm."""
-        ...
-
     def run(
         self,
-        data: BaseSource,
         model_params: _SerializedWeights,
         iterations: int,
     ) -> Tuple[_Weights, Optional[Dict[str, str]]]:
@@ -123,18 +116,10 @@ class _FederatedAveragingCompatibleWorker(_FederatedAveragingCompatibleAlgo, Pro
 
 @runtime_checkable
 class _FederatedAveragingCompatibleModeller(
-    _FederatedAveragingCompatibleAlgo, Protocol
+    _FederatedAveragingCompatibleAlgo, BaseCompatibleModellerAlgorithm, Protocol
 ):
     """Defines modeller-side algorithm compatibility."""
 
-    def initialise(
-        self,
-        task_id: Optional[str] = None,
-        **kwargs: Any,
-    ) -> None:
-        """Initialises the modeller-side algorithm."""
-        ...
-
     def run(
         self,
         iteration: int = 0,
@@ -283,7 +268,6 @@ class _ModellerSide(BaseModellerProtocol, _BaseFederatedAveragingMixIn):
         **kwargs: Any,
     ) -> List[Dict[str, float]]:
         """Receives updates and sends new parameters in a loop."""
-        self.algorithm.initialise(task_id=self.mailbox._task_id)
         self.perform_iterations_checks()
         initial_parameters = self.algorithm.run(update=None)
         logger.federated_info("Sending model parameters to pods...")
@@ -351,7 +335,7 @@ class _ModellerSide(BaseModellerProtocol, _BaseFederatedAveragingMixIn):
             if training_complete:
                 logger.info("Early stopping criterion met. Stopping training.")
                 break
-        await self.mailbox.send_task_complete_message()
+
         modeller_results = self.validation_results
         return modeller_results
 
@@ -407,14 +391,10 @@ class _WorkerSide(BaseWorkerProtocol, _BaseFederatedAveragingMixIn):
 
     async def run(
         self,
-        datasource: BaseSource,
-        pod_dp: Optional[DPPodConfig] = None,
         pod_vitals: Optional[_PodVitals] = None,
-        pod_identifier: Optional[str] = None,
         **kwargs: Any,
     ) -> Any:
         """Receives parameters and sends updates in a loop."""
-        self.algorithm.initialise(datasource=datasource, pod_dp=pod_dp)
         self.perform_iterations_checks()
         logger.debug("Waiting on initial parameters")
         serialized_model_params = await self._receive_parameters()
@@ -433,7 +413,7 @@ class _WorkerSide(BaseWorkerProtocol, _BaseFederatedAveragingMixIn):
             if pod_vitals:
                 pod_vitals.last_task_execution_time = time.time()
             parameter_update, validation_metrics = self.algorithm.run(
-                datasource, serialized_model_params, iterations
+                serialized_model_params, iterations
             )
 
             # Aggregator returns the difference between the new and old model parameters
@@ -458,7 +438,7 @@ class _WorkerSide(BaseWorkerProtocol, _BaseFederatedAveragingMixIn):
                     + "Stopping training.",
                 )
                 break
-        await self.mailbox.get_task_complete_update()
+
         self.algorithm.save_final_parameters(serialized_model_params)
 
 
