@@ -1,4 +1,4 @@
-"""Results Only protocol."""
+"""Private Set Intersection protocol."""
 from __future__ import annotations
 
 import time
@@ -14,7 +14,6 @@ from typing import (
 )
 
 from cryptography.hazmat.primitives.asymmetric.rsa import RSAPublicKey
-from marshmallow import fields
 import pandas as pd
 
 from bitfount.data.datasources.base_source import BaseSource
@@ -22,10 +21,10 @@ import bitfount.federated.algorithms.base as algorithms
 from bitfount.federated.encryption import _RSAEncryption
 from bitfount.federated.exceptions import PSINoDataSourceError
 from bitfount.federated.logging import _get_federated_logger
-from bitfount.federated.pod_vitals import _PodVitals
-from bitfount.federated.privacy.differential import DPPodConfig
 from bitfount.federated.protocols.base import (
     BaseCompatibleAlgoFactory,
+    BaseCompatibleModellerAlgorithm,
+    BaseCompatibleWorkerAlgorithm,
     BaseModellerProtocol,
     BaseProtocolFactory,
     BaseWorkerProtocol,
@@ -42,28 +41,23 @@ from bitfount.federated.transport.worker_transport import (
     _send_public_key,
     _WorkerMailbox,
 )
-from bitfount.types import T_FIELDS_DICT, T_NESTED_FIELDS
+from bitfount.types import T_NESTED_FIELDS
 from bitfount.utils import delegates
 
 if TYPE_CHECKING:
+    from bitfount.federated.pod_vitals import _PodVitals
     from bitfount.hub.api import BitfountHub
 
+
 logger = _get_federated_logger(__name__)
 
 
 @runtime_checkable
-class _PSICompatibleModeller(Protocol):
+class _PSICompatibleModeller(BaseCompatibleModellerAlgorithm, Protocol):
     """Defines modeller-side algorithm compatibility."""
 
-    def initialise(
-        self,
-        datasource: BaseSource,
-        public_key: RSAPublicKey,
-        columns_to_intersect: Optional[List[str]] = None,
-        table: Optional[str] = None,
-        **kwargs: Any,
-    ) -> List[int]:
-        """Initialises the modeller-side algorithm."""
+    def get_modeller_set(self, public_key: RSAPublicKey) -> List[int]:
+        """Returns the modeller-side set."""
         ...
 
     def run(self, pod_set: List[int], modeller_set: List[int]) -> pd.DataFrame:
@@ -72,20 +66,10 @@ class _PSICompatibleModeller(Protocol):
 
 
 @runtime_checkable
-class _PSICompatibleWorker(Protocol):
+class _PSICompatibleWorker(BaseCompatibleWorkerAlgorithm, Protocol):
     public_key: RSAPublicKey
 
-    def initialise(
-        self,
-        datasource: BaseSource,
-        columns_to_intersect: Optional[List[str]] = None,
-        table: Optional[str] = None,
-        **kwargs: Any,
-    ) -> List[int]:
-        """Initialises the worker-side algorithm."""
-        ...
-
-    def run(self, modeller_set: List[int]) -> List[int]:
+    def run(self, modeller_set: List[int]) -> Tuple[List[int], List[int]]:
         """Runs the worker-side algorithm."""
         ...
 
@@ -101,8 +85,6 @@ class _ModellerSide(BaseModellerProtocol):
         *,
         algorithm: _PSICompatibleModeller,
         datasource: Optional[BaseSource] = None,
-        columns_to_intersect: Optional[List[str]] = None,
-        table: Optional[str] = None,
         mailbox: _ModellerMailbox,
         **kwargs: Any,
     ):
@@ -113,8 +95,6 @@ class _ModellerSide(BaseModellerProtocol):
                 "Please provide a datasource and try again."
             )
         self.datasource = datasource
-        self.columns_to_intersect = columns_to_intersect
-        self.table = table
         super().__init__(algorithm=algorithm, mailbox=mailbox, **kwargs)
 
     async def _send_psi_data_modeller(self, dataset: List[int]) -> None:
@@ -149,22 +129,28 @@ class _ModellerSide(BaseModellerProtocol):
         )
         return psi_datasets_int
 
+    def initialise(
+        self,
+        task_id: Optional[str] = None,
+        **kwargs: Any,
+    ) -> None:
+        """Override base initialise method to pass datasource during initialisation."""
+        self.algorithm.initialise(
+            datasource=self.datasource,
+            task_id=task_id,
+            **kwargs,
+        )
+
     async def run(
         self,
         **kwargs: Any,
     ) -> pd.DataFrame:
         """Runs Modeller side of the protocol."""
         public_key = await self._receive_public_key()
-        modeller_input = self.algorithm.initialise(
-            public_key=public_key,
-            datasource=self.datasource,
-            columns_to_intersect=self.columns_to_intersect,
-            table=self.table,
-        )
+        modeller_input = self.algorithm.get_modeller_set(public_key)
         await self._send_psi_data_modeller(modeller_input)
         pod_set, modeller_set = await self._receive_psi_datasets_from_workers()
         modeller_results = self.algorithm.run(pod_set, modeller_set)
-        await self.mailbox.send_task_complete_message()
         return modeller_results
 
 
@@ -177,13 +163,9 @@ class _WorkerSide(BaseWorkerProtocol):
         self,
         *,
         algorithm: _PSICompatibleWorker,
-        columns_to_intersect: Optional[List[str]] = None,
-        table: Optional[str] = None,
         mailbox: _WorkerMailbox,
         **kwargs: Any,
     ):
-        self.columns_to_intersect = columns_to_intersect
-        self.table = table
         super().__init__(algorithm=algorithm, mailbox=mailbox, **kwargs)
 
     async def _receive_modeller_psi_dataset(self) -> List[int]:
@@ -209,31 +191,25 @@ class _WorkerSide(BaseWorkerProtocol):
 
     async def run(
         self,
-        datasource: BaseSource,
-        pod_dp: Optional[DPPodConfig] = None,
         pod_vitals: Optional[_PodVitals] = None,
-        pod_identifier: Optional[str] = None,
         **kwargs: Any,
     ) -> None:
         """Runs Worker side of the protocol."""
-        await self._send_public_key_to_modeller(self.algorithm.public_key)
-        pod_set = self.algorithm.initialise(
-            datasource=datasource,
-            columns_to_intersect=self.columns_to_intersect,
-            table=self.table,
-        )
-        modeller_set = await self._receive_modeller_psi_dataset()
         if pod_vitals:
             pod_vitals.last_task_execution_time = time.time()
-        hashed_modeller_set = self.algorithm.run(modeller_set)
+        await self._send_public_key_to_modeller(self.algorithm.public_key)
+        modeller_set = await self._receive_modeller_psi_dataset()
+        pod_set, hashed_modeller_set = self.algorithm.run(modeller_set)
         await self._send_psi_data_to_modeller((pod_set, hashed_modeller_set))
-        await self.mailbox.get_task_complete_update()
 
 
 @runtime_checkable
 class _PSICompatibleAlgoFactory_(BaseCompatibleAlgoFactory, Protocol):
     """Defines algo factory compatibility."""
 
+    pod_columns: Optional[List[str]]
+    pod_table: Optional[str]
+
     def modeller(self, **kwargs: Any) -> _PSICompatibleModeller:
         """Creates a modeller-side algorithm."""
         ...
@@ -253,29 +229,11 @@ class PrivateSetIntersection(BaseProtocolFactory):
     Args:
         algorithm: The algorithm to run.
         datasource: The modeller's datasource.
-        datasource_columns: The modeller's columns from their datasource
-            on which the private set intersection will be computed as a
-            list of strings. Defaults to None.
-        datasource_table: The modeller's table from their datasource,
-            if the datasource is multitable, on which the private set
-            intersection will be computed as a string. Defaults to None.
-        pod_columns: The pod's columns from their datasource
-            on which the private set intersection will be computed as a
-            list of strings. Defaults to None.
-        pod_table: The pod's table from their datasource,
-            if the datasource is multitable, on which the private set
-            intersection will be computed as a string. Defaults to None.
 
     Attributes:
         name: The name of the protocol.
         algorithm: The algorithm to run. This must be compatible with the
             `PrivateSetIntersection` protocol.
-        pod_columns: The pod's columns from their datasource
-            on which the private set intersection will be computed as a
-            list of strings. Defaults to None.
-        pod_table: The pod's table from their datasource,
-            if the datasource is multitable, on which the private set
-            intersection will be computed as a string. Defaults to None.
 
     Raises:
         TypeError: If the `algorithm` is not compatible with the protocol.
@@ -284,11 +242,6 @@ class PrivateSetIntersection(BaseProtocolFactory):
     """
 
     algorithm: _PSICompatibleAlgoFactory_
-    # We only need to serialize the pod columns & table
-    fields_dict: ClassVar[T_FIELDS_DICT] = {
-        "pod_columns": fields.List(fields.String(default=None), allow_none=True),
-        "pod_table": fields.String(allow_none=True),
-    }
     nested_fields: ClassVar[T_NESTED_FIELDS] = {"algorithm": algorithms.registry}
 
     def __init__(
@@ -296,17 +249,9 @@ class PrivateSetIntersection(BaseProtocolFactory):
         *,
         algorithm: _PSICompatibleAlgoFactory_,
         datasource: Optional[BaseSource] = None,
-        datasource_columns: Optional[List[str]] = None,
-        datasource_table: Optional[str] = None,
-        pod_columns: Optional[List[str]] = None,
-        pod_table: Optional[str] = None,
         **kwargs: Any,
     ) -> None:
         self.datasource = datasource
-        self._modeller_cols = datasource_columns
-        self._modeller_table = datasource_table
-        self.pod_columns = pod_columns
-        self.pod_table = pod_table
         super().__init__(algorithm=algorithm, **kwargs)
 
     @classmethod
@@ -330,8 +275,6 @@ class PrivateSetIntersection(BaseProtocolFactory):
         return _ModellerSide(
             algorithm=algorithm,
             datasource=self.datasource,
-            columns_to_intersect=self._modeller_cols,
-            table=self._modeller_table,
             mailbox=mailbox,
             **kwargs,
         )
@@ -342,8 +285,6 @@ class PrivateSetIntersection(BaseProtocolFactory):
         """Returns the worker side of the PrivateSetIntersection protocol."""
         return _WorkerSide(
             algorithm=self.algorithm.worker(hub=hub),
-            columns_to_intersect=self.pod_columns,
-            table=self.pod_table,
             mailbox=mailbox,
             **kwargs,
         )
