@@ -52,7 +52,6 @@ from bitfount.federated.transport.message_service import (
     _BitfountMessageType,
     _MessageEncryption,
     _MessageService,
-    msgpackext_decode,
     msgpackext_encode,
 )
 from bitfount.utils.concurrency_utils import (
@@ -408,8 +407,12 @@ class _BaseMailbox:
         through faster than a handler can be registered. If not, it is passed
         to the default handler.
         """
-        running_dispatch_tasks: Set[Task] = set()
-        done_dispatch_tasks: Set[Task] = set()
+        running_dispatch_tasks: Set[
+            Task[List[Union[ConcurrentFuture, _PriorityHandler]]]
+        ] = set()
+        done_dispatch_tasks: Set[
+            Task[List[Union[ConcurrentFuture, _PriorityHandler]]]
+        ] = set()
 
         # Guarantee that only one listener is listening at a time.
         # We can set there to be NO_POLLING despite the dangers as
@@ -452,7 +455,15 @@ class _BaseMailbox:
                     for task in done_dispatch_tasks:
                         # If there was an exception in the task dispatch,
                         # this will reraise it here.
-                        task.result()
+                        handler_results = task.result()
+                        for result in handler_results:
+                            # If there was an exception in the actual handler,
+                            # this will reraise it here
+                            if isinstance(result, _PriorityHandler):
+                                await result.result()
+                            else:
+                                result.result()
+
                         logger.debug(
                             f"Dispatch task {task.get_name()} is done"
                             f" in mailbox {self.mailbox_id}"
@@ -483,7 +494,7 @@ class _BaseMailbox:
                 raise MessageRetrievalError(
                     f"An error occurred when trying to communicate"
                     f" with the messaging service: {err}"
-                )
+                ) from err
             finally:
                 # Cancel outstanding dispatch tasks
                 if len(running_dispatch_tasks) > 0:
@@ -1163,20 +1174,6 @@ async def _send_aes_encrypted_message(
     )
 
 
-def _decrypt_aes_message(message: _BitfountMessage, aes_encryption_key: bytes) -> Any:
-    """Decrypt AES encrypted message.
-
-    Args:
-        message: Encrypted message to decrypt.
-        aes_encryption_key: The AES key to use to decrypt.
-
-    Returns:
-        Decrypted message body.
-    """
-    body = _MessageEncryption.decrypt_incoming_message(message.body, aes_encryption_key)
-    return msgpack.loads(body, ext_hook=msgpackext_decode)
-
-
 # Type variable for return type below.
 _R = TypeVar("_R")
 
