@@ -57,6 +57,10 @@ from bitfount.federated.transport.protos.messages_pb2 import (
     LargeStorageRequest,
     PodData,
     SuccessResponse,
+    TaskMetadata,
+    TaskTransferMetadata,
+    TaskTransferRequest,
+    TaskTransferRequests,
 )
 from bitfount.federated.transport.protos.messages_pb2_grpc import MessageServiceStub
 from bitfount.federated.transport.types import CommunicationDetails
@@ -96,7 +100,7 @@ def _is_s3_netloc(netloc: str) -> bool:
 # When messages are packaged on the message service we attach some metadata,
 # the metadata will make them larger, so the python code needs a lower limit.
 # So this limit ensures that our message service will accept the raw messages.
-_SMALL_MESSAGE_UPPER_LIMIT_SIZE_BYTES: int = 100 * 1024  # 100kb
+_SMALL_MESSAGE_UPPER_LIMIT_SIZE_BYTES: int = 128 * 1024  # 128kb
 
 # Maximum storage request size in message service: 3gb. This should match the value
 # in bitfount/message-service:src/messaging/queue.service.ts
@@ -216,6 +220,12 @@ class _BitfountMessageType(Enum):
     PSI_DATASET = GrpcBitfountMessage.PSI_DATASET
     # task start message modeller->worker
     TASK_START = GrpcBitfountMessage.TASK_START
+    # Number of batches for batched execution worker->modeller
+    NUMBER_OF_BATCHES = GrpcBitfountMessage.NUMBER_OF_BATCHES
+    # Model prompt message modeller->worker
+    MODEL_PROMPT = GrpcBitfountMessage.MODEL_PROMPT
+    # Model response message worker->modeller
+    MODEL_RESPONSE = GrpcBitfountMessage.MODEL_RESPONSE
 
 
 class _LargeObjectRequestHandler:
@@ -471,6 +481,8 @@ class _AutoRetryMessageServiceStub:
 
         self.PodConnect = _auto_retry_grpc(orig.PodConnect)
         self.SetupTaskMailboxes = _auto_retry_grpc(orig.SetupTaskMailboxes)
+        self.SetupTask = _auto_retry_grpc(orig.SetupTask)
+        self.InitiateTask = _auto_retry_grpc(orig.InitiateTask)
         self.SendBitfountMessage = _auto_retry_grpc(orig.SendBitfountMessage)
         self.GetLargeObjectStorage = _auto_retry_grpc(orig.GetLargeObjectStorage)
 
@@ -558,7 +570,9 @@ class _MessageService:
         """Returns metadata required for sending a BitfountMessage."""
         return self._session.message_service_metadata
 
-    async def connect_pod(self, pod_name: str) -> str:
+    async def connect_pod(
+        self, pod_name: str, dataset_names: Optional[List[str]] = None
+    ) -> str:
         """Make the message service aware of the Pod.
 
         This ensures that the correct queue is set up for this pod,
@@ -566,6 +580,8 @@ class _MessageService:
 
         Args:
             pod_name: The name of the pod (the part after 'username/')
+            dataset_names: Optional. The name of the datasets, in case there is
+                more than one (also the part after 'username/').
 
         Returns:
             The created mailbox ID for this pod.
@@ -593,6 +609,7 @@ class _MessageService:
                     totalMemoryBytes=total_physical_memory_bytes,
                     gpuCount=available_gpus,
                     gpuName=gpu_name,
+                    datasetNames=dataset_names,
                 ),
                 metadata=self.metadata,
             )
@@ -602,7 +619,7 @@ class _MessageService:
             raise BitfountMessageServiceError(
                 "Unable to connect pod to message service."
                 " See debug logs for more details."
-            )
+            ) from e
 
         # If we've not received a successful response (but no other RPC error
         # happened) then raise it here.
@@ -670,8 +687,97 @@ class _MessageService:
             raise BitfountMessageServiceError(
                 "Unable to setup communication with target pods."
                 " See debug logs for more details."
+            ) from e
+
+        # communication_details.podMailboxIds is actually the _worker_ mailbox_ids
+        return CommunicationDetails(
+            communication_details.mailboxId,
+            dict(communication_details.podMailboxIds),
+            communication_details.taskId,
+        )
+
+    async def setup_task(
+        self,
+        tasks_per_pod: Dict[str, bytes],
+        task_metadata: TaskMetadata,
+        project_id: Optional[str] = None,
+    ) -> CommunicationDetails:
+        """Called by the modeller to set up communication channels with specified pods.
+
+        Args:
+            tasks_per_pod: Mapping of pod identifiers to AES Encrypted messages.
+            task_metadata: Metadata about the encrypted tasks
+            project_id: ID of the project the task is associated with
+
+        Returns:
+            A tuple of mailbox IDs used for communications:
+                - mailbox_id (str): The modeller's mailbox ID
+                - worker_mailbox_ids (dict): Mapping from Pod ID to Worker Mailbox ID
+                - task_id (str): The unique task ID for the task
+
+        Raises:
+            BitfountMessageServiceError: If an error occurs when communicating
+                                         with the message service.
+        """
+        logger.debug(f"Setting up task configuration with: '{tasks_per_pod.keys()}'")
+        try:
+            # All tasks are transferred via Blob storage,
+            # so we need to request the appropriate amounts of storage.
+            # This can differ per pod as the tasks are encrypted.
+            task_sizes = []
+            for pod_identifier, aes_encrypted_task in tasks_per_pod.items():
+                task_sizes.append(
+                    TaskTransferRequest(
+                        podIdentifier=pod_identifier,
+                        contentSize=_get_packed_data_object_size(aes_encrypted_task),
+                    )
+                )
+
+            # Request the storage, and get a Task ID
+            task_upload_metadata: TaskTransferMetadata = await (
+                await self.stub
+            ).SetupTask(
+                TaskTransferRequests(podTasks=task_sizes, projectId=project_id),
+                metadata=self.metadata,
             )
 
+            # Upload each encrypted task to the appropriately sized location
+            tasks = []
+            for storage_metadata in task_upload_metadata.taskStorage:
+                await _LargeObjectRequestHandler.upload_large_object(
+                    cast(_S3PresignedPOSTURL, storage_metadata.uploadUrl),
+                    cast(_S3PresignedPOSTFields, storage_metadata.uploadFields),
+                    tasks_per_pod[storage_metadata.podIdentifier],
+                )
+                # Prepare the message that will be sent to each pod containing the task
+                tasks.append(
+                    BitfountTask(
+                        podIdentifier=storage_metadata.podIdentifier,
+                        taskURL=msgpack.dumps(
+                            storage_metadata.downloadUrl, default=msgpackext_encode
+                        ),
+                    )
+                )
+
+            # Send the URL for each task on blob storage to each pod
+            communication_details: GrpcCommunicationDetails = await (
+                await self.stub
+            ).InitiateTask(
+                BitfountTasks(
+                    tasks=tasks,
+                    taskId=task_upload_metadata.taskId,
+                    projectId=project_id,
+                    taskMetadata=task_metadata,
+                ),
+                metadata=self.metadata,
+            )
+        except (RpcError, RequestException) as e:
+            logger.debug(e, exc_info=True)
+            raise BitfountMessageServiceError(
+                "Unable to start task with target pods."
+                " See debug logs for more details."
+            ) from e
+
         # communication_details.podMailboxIds is actually the _worker_ mailbox_ids
         return CommunicationDetails(
             communication_details.mailboxId,
@@ -834,7 +940,7 @@ class _MessageService:
                 raise BitfountMessageServiceError(
                     "Issue retrieving message from mailbox."
                     " See debug log for details."
-                )
+                ) from ex
 
         b_message = await _BitfountMessage.from_rpc(message)
         logger.debug(
@@ -951,7 +1057,7 @@ class _MessageService:
             raise BitfountMessageServiceError(
                 "Encountered problem when sending message."
                 " See debug logs for more details."
-            )
+            ) from e
 
     async def _maybe_upload_to_large_object_storage(
         self, message: _BitfountMessage
@@ -1040,7 +1146,7 @@ class _MessageService:
             raise BitfountMessageServiceError(
                 "Unable to acquire large object storage."
                 " See debug logs for more details."
-            )
+            ) from e
 
     @staticmethod
     def _save_object_to_local_storage(message: _BitfountMessage) -> bytes:
