@@ -5,7 +5,6 @@ from dataclasses import dataclass
 import logging
 from typing import (
     Any,
-    Callable,
     Dict,
     Final,
     Iterable,
@@ -24,12 +23,14 @@ from cryptography.hazmat.primitives.asymmetric.rsa import RSAPublicKey
 from grpc import RpcError
 
 from bitfount import config
+from bitfount.config import BITFOUNT_DEFAULT_BATCHED_EXECUTION
 from bitfount.federated.encryption import _AESEncryption
-from bitfount.federated.exceptions import BitfountTaskStartError
+from bitfount.federated.exceptions import BitfountTaskStartError, TaskAbortError
 from bitfount.federated.logging import _federate_logger, _get_federated_logger
 from bitfount.federated.transport.base_transport import (
     Handler,
     MessageRetrievalError,
+    SyncHandler,
     _BaseMailbox,
     _send_aes_encrypted_message,
 )
@@ -40,6 +41,7 @@ from bitfount.federated.transport.message_service import (
     _DecryptedBitfountMessage,
     _MessageService,
 )
+from bitfount.federated.transport.protos.messages_pb2 import TaskMetadata
 from bitfount.federated.transport.types import (
     _OIDCAuthFlowResponse,
     _OIDCClientID,
@@ -115,9 +117,12 @@ class _ModellerMailbox(_BaseMailbox):
         self._task_id = task_id
 
         self.accepted_worker_mailboxes: Dict[str, _WorkerMailboxDetails] = {}
-
+        self.pods_ready: bool = False
+        self._num_pods_ready: int = 0
         self._setup_federated_logging()
         self._setup_online_status_handler()
+        self._setup_task_abort_handler()
+        self._setup_task_start_handler()
 
     @property
     def task_id(self) -> str:
@@ -134,6 +139,9 @@ class _ModellerMailbox(_BaseMailbox):
         pod_public_keys: Mapping[str, RSAPublicKey],
         task_request_msg_gen: _TaskRequestMessageGenerator,
         message_service: _MessageService,
+        project_id: Optional[str] = None,
+        run_on_new_data_only: bool = False,
+        batched_execution: Optional[bool] = None,
     ) -> _ModellerMailbox:
         """Sends task requests, such as training requests, to pods.
 
@@ -147,15 +155,25 @@ class _ModellerMailbox(_BaseMailbox):
             task_request_msg_gen: A callable which will generate a task request message
                 appropriate to the chosen verification method.
             message_service: The underlying message service to use.
+            project_id: The project Id the task belongs to. Defaults to None.
+            run_on_new_data_only: Whether to run the task on new datapoints only.
+                Defaults to False.
+            batched_execution: Whether to run the task in batched execution mode.
+                Defaults to False.
 
         Returns:
             The created modeller mailbox for this task.
         """
+        if batched_execution is None:
+            batched_execution = BITFOUNT_DEFAULT_BATCHED_EXECUTION
         modeller_mailbox_id, worker_mailboxes, task_id = await cls._send_task_requests(
             serialized_protocol,
             pod_public_keys,
             task_request_msg_gen=task_request_msg_gen,
             message_service=message_service,
+            project_id=project_id,
+            run_on_new_data_only=run_on_new_data_only,
+            batched_execution=batched_execution,
         )
         modeller_mailbox = cls(
             mailbox_id=modeller_mailbox_id,
@@ -172,6 +190,9 @@ class _ModellerMailbox(_BaseMailbox):
         pod_public_keys: Mapping[str, RSAPublicKey],
         task_request_msg_gen: _TaskRequestMessageGenerator,
         message_service: _MessageService,
+        project_id: Optional[str],
+        run_on_new_data_only: bool = False,
+        batched_execution: Optional[bool] = None,
     ) -> Tuple[str, Dict[str, _WorkerMailboxDetails], str]:
         """Manage sending of task requests, such as training requests, to pods.
 
@@ -182,6 +203,11 @@ class _ModellerMailbox(_BaseMailbox):
             task_request_msg_gen: A callable which will generate a task request message
                 appropriate to the chosen verification method.
             message_service: The underlying message service to use.
+            project_id: The project Id the task belongs to.
+            run_on_new_data_only: Whether to run the task on new datapoints only.
+                Defaults to False.
+            batched_execution: Whether to run the task in batched execution mode.
+                Defaults to False.
 
         Returns:
             Tuple of:
@@ -191,7 +217,8 @@ class _ModellerMailbox(_BaseMailbox):
         """
         # Shorthand the pod identifiers for ease of use
         pod_identifiers: List[str] = list(pod_public_keys)
-
+        if batched_execution is None:
+            batched_execution = BITFOUNT_DEFAULT_BATCHED_EXECUTION
         # Generate encryption keys and then task request messages for each pod.
         aes_key_per_pod = {
             pod_identifier: _AESEncryption.generate_key()
@@ -203,6 +230,9 @@ class _ModellerMailbox(_BaseMailbox):
                 pod_identifiers,
                 aes_key_per_pod[pod_identifier],
                 pod_public_key,
+                project_id,
+                run_on_new_data_only,
+                batched_execution,
             )
             for pod_identifier, pod_public_key in pod_public_keys.items()
         }
@@ -214,8 +244,10 @@ class _ModellerMailbox(_BaseMailbox):
                 modeller_mailbox_id,
                 worker_mailbox_ids,
                 task_id,
-            ) = await message_service.setup_communication_with_pods(
-                task_request_per_pod
+            ) = await message_service.setup_task(
+                task_request_per_pod,
+                TaskMetadata(protocol=serialized_protocol["class_name"]),
+                project_id,
             )
             logger.info(f"Sent task requests to {pod_identifiers}")
         except RpcError as err:
@@ -224,7 +256,7 @@ class _ModellerMailbox(_BaseMailbox):
             )
             raise BitfountTaskStartError(
                 f"Failed to start task with pods: {pod_identifiers}"
-            )
+            ) from err
 
         return (
             modeller_mailbox_id,
@@ -278,7 +310,9 @@ class _ModellerMailbox(_BaseMailbox):
                     f"Error receiving responses from all pods to the OIDC phase "
                     f"of the task request: {err}"
                 )
-                raise BitfountTaskStartError("Failed to start task with all pods.")
+                raise BitfountTaskStartError(
+                    "Failed to start task with all pods."
+                ) from err
 
         if not oidc_client_ids:
             logger.error("No OIDC client id retreived from message handler")
@@ -363,7 +397,9 @@ class _ModellerMailbox(_BaseMailbox):
                     f"Error receiving responses from all pods to the task request: "
                     f"{err}"
                 )
-                raise BitfountTaskStartError("Failed to start task with all pods.")
+                raise BitfountTaskStartError(
+                    "Failed to start task with all pods."
+                ) from err
 
         return saml_challenges
 
@@ -414,7 +450,9 @@ class _ModellerMailbox(_BaseMailbox):
                     f"Error receiving responses from all pods to the task request: "
                     f"{err}"
                 )
-                raise BitfountTaskStartError("Failed to start task with all pods.")
+                raise BitfountTaskStartError(
+                    "Failed to start task with all pods."
+                ) from err
 
         accepted_mailbox_details = await self._handle_task_responses(response_messages)
 
@@ -542,7 +580,7 @@ class _ModellerMailbox(_BaseMailbox):
 
         Note: The message is not important here, the message type is.
         """
-        logger.debug(f"Sending TASK_COMPLETE message to workers from {self.mailbox_id}")
+        logger.info(f"Sending TASK_COMPLETE message to workers from {self.mailbox_id}")
         await self._send_to_all_pods_aes_encrypt(
             None, _BitfountMessageType.TASK_COMPLETE
         )
@@ -592,6 +630,12 @@ class _ModellerMailbox(_BaseMailbox):
     def _setup_federated_logging(self) -> None:
         """Set up federated logging."""
         _federate_logger(self)
+        self.register_handler(
+            _BitfountMessageType.LOG_MESSAGE, self._get_log_message_handler()
+        )
+
+    def _get_log_message_handler(self) -> SyncHandler:
+        """Create the appropriate handler for LOG_MESSAGE messages."""
 
         def log_message_handler(message: _BitfountMessage) -> None:
             """Locally logs the log message that has been received from a pod."""
@@ -599,14 +643,26 @@ class _ModellerMailbox(_BaseMailbox):
                 message
             )
             log_message: _JSONDict = log_message_wrapper.body
+
             # We prepend the name of the pod to the log message
-            log_message["msg"] = f"{message.sender} Pod: {log_message['msg']}"
+            log_message["msg"] = f"<FROM POD {message.sender}>: {log_message['msg']}"
+
+            # Modify processName and threadName to indicate these are non-local
+            try:
+                log_message["processName"] = f"<{log_message['processName']}>"
+            except KeyError:
+                pass
+            try:
+                log_message["threadName"] = f"<{log_message['threadName']}>"
+            except KeyError:
+                pass
+
             # We remove the `federated` key to avoid recursively sending a federated
             # log message on both the Modeller and Worker sides
             log_message.pop("federated")
             logger.handle(logging.makeLogRecord(log_message))
 
-        self.register_handler(_BitfountMessageType.LOG_MESSAGE, log_message_handler)
+        return log_message_handler
 
     def _setup_online_status_handler(self) -> None:
         """Respond to online status requests from Pods."""
@@ -637,6 +693,85 @@ class _ModellerMailbox(_BaseMailbox):
             high_priority=True,
         )
 
+    def _setup_task_abort_handler(self) -> None:
+        """Process TASK_ABORT message from Pods."""
+
+        async def task_abort_handler(message: _BitfountMessage) -> None:
+            """Abort the task."""
+            task_abort_message_body: Optional[str] = self._decrypt_message(message).body
+
+            error_message = f"Received TASK_ABORT message from {message.sender}"
+            if task_abort_message_body is not None:
+                error_message = f"{error_message}: {task_abort_message_body}"
+
+            logger.info(error_message)
+            raise TaskAbortError(error_message)
+
+        self.register_handler(
+            _BitfountMessageType.TASK_ABORT,
+            task_abort_handler,
+        )
+
+    def _setup_task_start_handler(self) -> None:
+        """Wait for TASK_START message from all Pods."""
+
+        def task_start_handler(message: _BitfountMessage) -> None:
+            """Simply keep track of how many Pods have sent a TASK_START message."""
+            self._num_pods_ready += 1
+            if self._num_pods_ready == len(self.accepted_worker_mailboxes):
+                logger.info("All Pods are ready to start the task.")
+                self.pods_ready = True
+
+        self.register_handler(
+            _BitfountMessageType.TASK_START,
+            task_start_handler,
+        )
+
+    async def get_num_batches_message(self, timeout: Optional[int] = None) -> int:
+        """Get number of batches from worker for batched execution.
+
+        This is intended to be used for batched execution, where the number of batches
+        is not known in advance by the modeller so the modeller must get it from the
+        worker. Batched execution is only supported in cases where there is only one
+        worker.
+
+        Args:
+            modeller_mailbox: The modeller mailbox.
+            timeout: The timeout for the request.
+
+        Returns:
+            A number of batches.
+
+        Raises:
+            ValueError: If the number of responses is not 1.
+        """
+        num_batches_list: List[int] = []
+
+        def batched_execution_handler(message: _BitfountMessage) -> None:
+            logger.debug(
+                f"Receiving number of batches update from worker {message.sender}"
+            )
+            # Deliberate access to private method here as that method shouldn't be used
+            # in any other context than transport layer access.
+            # noinspection PyProtectedMember
+            num_batches: int = self._decrypt_message(message).body
+            num_batches_list.append(num_batches)
+
+        with _AsyncMultipleResponsesHandler(
+            handler=batched_execution_handler,
+            message_types=_BitfountMessageType.NUMBER_OF_BATCHES,
+            mailbox=self,
+            responders=self.accepted_worker_mailboxes.keys(),
+        ) as response_handler:
+            await response_handler.wait_for_responses(timeout=timeout)
+
+        if len(num_batches_list) != 1:
+            raise ValueError(
+                f"Expected one response from worker for number of batches, "
+                f"got {len(num_batches_list)}"
+            )
+        return num_batches_list[0]
+
     ##################################
     # End Task Running Phase Methods #
     ##################################
@@ -654,6 +789,16 @@ async def _send_model_parameters(
     )
 
 
+async def _send_prompt(prompt: str, modeller_mailbox: _ModellerMailbox) -> None:
+    """Sends model parameters to the workers."""
+    # Deliberate access to private method here as that method shouldn't be used in
+    # any other context than transport layer access.
+    # noinspection PyProtectedMember
+    await modeller_mailbox._send_to_all_pods_aes_encrypt(
+        prompt, _BitfountMessageType.MODEL_PROMPT
+    )
+
+
 async def _send_psi_dataset_modeller(
     dataset: List[str], modeller_mailbox: _ModellerMailbox
 ) -> None:
@@ -666,7 +811,7 @@ async def _send_psi_dataset_modeller(
 def _psi_dataset_handler(
     modeller_mailbox: _ModellerMailbox,
     psi_datasets_str: List[Tuple[List[str], List[str]]],
-) -> Callable[[_BitfountMessage], None]:
+) -> SyncHandler:
     """Handle receiving of PSI datasets from workers.
 
     Appends them to the `psi_datasets_str` list.
@@ -674,8 +819,9 @@ def _psi_dataset_handler(
     Note that there is no notion of "worker order" in this list, the elements are
     in the order they are received.
     """
-    # Create light-weight handler to append to shared list
+
     def psi_dataset_handler(message: _BitfountMessage) -> None:
+        # Create light-weight handler to append to shared list
         logger.debug(f"Receiving psi dataset from worker {message.sender}")
         # Deliberate access to private method here as that method shouldn't be used in
         # any other context than transport layer access.
@@ -710,7 +856,7 @@ async def _get_psi_datasets_from_workers(
 
 def _public_key_handler(
     modeller_mailbox: _ModellerMailbox, public_key: List[bytes]
-) -> Callable[[_BitfountMessage], None]:
+) -> SyncHandler:
     """Public key handler.
 
     Will mutate the passed in sequence by appending responses to it.
@@ -760,7 +906,7 @@ async def _get_public_key(
 def _training_metrics_handler(
     modeller_mailbox: _ModellerMailbox,
     training_metrics: MutableSequence[Mapping[str, str]],
-) -> Callable[[_BitfountMessage], None]:
+) -> SyncHandler:
     """Training metrics handler.
 
     Will mutate the passed in sequence by appending responses to it.
@@ -811,7 +957,7 @@ async def _get_training_metrics_from_workers(
 def _parameter_updates_handler(
     modeller_mailbox: _ModellerMailbox,
     weight_updates: MutableMapping[str, _SerializedWeights],
-) -> Callable[[_BitfountMessage], None]:
+) -> SyncHandler:
     """Parameter update handler.
 
     Will mutate the passed in mapping by appending responses to it.
@@ -855,3 +1001,51 @@ async def _get_parameter_updates_from_workers(
         await response_handler.wait_for_responses(timeout=timeout)
 
     return weight_updates
+
+
+def _model_responses_handler(
+    modeller_mailbox: _ModellerMailbox,
+    model_responses: MutableMapping[str, List[Dict[str, str]]],
+) -> SyncHandler:
+    """Model response handler.
+
+    Will mutate the passed in mapping by appending responses to it.
+    """
+
+    def model_responses_handler(message: _BitfountMessage) -> None:
+        logger.debug(f"Receiving model response from worker {message.sender}")
+        # Deliberate access to private method here as that method shouldn't be used in
+        # any other context than transport layer access.
+        # noinspection PyProtectedMember
+        model_response = modeller_mailbox._decrypt_message(message).body
+        sender = message.sender
+        model_responses[sender] = model_response
+
+    return model_responses_handler
+
+
+async def _get_model_responses_from_workers(
+    modeller_mailbox: _ModellerMailbox, timeout: Optional[int] = None
+) -> Dict[str, List[Dict[str, str]]]:
+    """Get model responses from workers.
+
+    Args:
+        modeller_mailbox: The modeller mailbox.
+        timeout: The timeout for the request.
+
+    Returns:
+        A dictionary of the form {worker_name: model_response}.
+    """
+    model_responses: Dict[str, List[Dict[str, str]]] = {}
+    model_responses_handler = _model_responses_handler(
+        modeller_mailbox, model_responses
+    )
+    with _AsyncMultipleResponsesHandler(
+        handler=model_responses_handler,
+        message_types=_BitfountMessageType.MODEL_RESPONSE,
+        mailbox=modeller_mailbox,
+        responders=modeller_mailbox.accepted_worker_mailboxes.keys(),
+    ) as response_handler:
+        await response_handler.wait_for_responses(timeout=timeout)
+
+    return model_responses
