@@ -3,6 +3,7 @@ from abc import ABC, abstractmethod
 from datetime import datetime, timedelta, timezone
 from functools import cached_property
 import json
+from json import JSONDecodeError
 import logging
 from pathlib import Path
 import time
@@ -33,11 +34,13 @@ _STAGING_AUTH_DOMAIN: Final = "auth.staging.bitfount.com"
 _DEVELOPMENT_AUTH_DOMAIN: Final = (
     "auth.staging.bitfount.com"  # this is part of the staging tenant on auth0
 )
+_SANDBOX_AUTH_DOMAIN: Final = "auth.staging.bitfount.com"
 
 # TODO: [BIT-356] potentially remove these client ids from the codebase
 _PRODUCTION_CLIENT_ID: Final = "8iCJ33Kp6hc9ofrXTzr5GLxMRHWrlzZO"
 _STAGING_CLIENT_ID: Final = "Wk4XZHDKfY8F3OYcKdagIHETt6JYwX08"
 _DEVELOPMENT_CLIENT_ID: Final = "MP8oao6gcJd4jARwzJiJlEiK59ZeLCt3"
+_SANDBOX_CLIENT_ID: Final = "Wk4XZHDKfY8F3OYcKdagIHETt6JYwX08"  # same as staging
 
 _SCOPES: Final = "profile openid offline_access"
 _HUB_API_IDENTIFIER: Final = (
@@ -223,9 +226,10 @@ class ExternallyManagedJWTHandler(AuthenticationHandler):
 
         Calls the hook provided on object creation to retrieve a new token.
         """
-        token, expires = self._get_token()
-        self._jwt = token
-        self._expires = expires
+        if not self.authenticated:
+            token, expires = self._get_token()
+            self._jwt = token
+            self._expires = expires
 
     @property
     def authenticated(self) -> bool:
@@ -388,7 +392,7 @@ class DeviceCodeFlowHandler(AuthenticationHandler):
         if refresh:
             # See: https://auth0.com/docs/api/authentication?http#refresh-token
             # If refreshing, must have refresh_token. Reassure mypy.
-            assert self.refresh_token is not None  # nosec[assert_used]
+            assert self.refresh_token is not None  # nosec assert_used
             refresh_request_data: _TokenRefreshRequestDict = {
                 "client_id": self._client_id,
                 "grant_type": "refresh_token",
@@ -399,7 +403,7 @@ class DeviceCodeFlowHandler(AuthenticationHandler):
             # See: https://auth0.com/docs/api/authentication?http#device-authorization-flow48  # noqa: B950
             # device_code must already be set by the time we are calling this.
             # Reassure mypy.
-            assert self.device_code is not None  # nosec[assert_used]
+            assert self.device_code is not None  # nosec assert_used
             request_data: _DeviceAccessTokenRequestDict = {
                 "client_id": self._client_id,
                 "grant_type": _DEVICE_CODE_GRANT_TYPE,
@@ -413,7 +417,7 @@ class DeviceCodeFlowHandler(AuthenticationHandler):
         Saves all fields that are necessary to reproduce this object to a file.
         """
         # This will be set by the time this is called. Reassure mypy.
-        assert self.access_token_expires_at is not None  # nosec[assert_used]
+        assert self.access_token_expires_at is not None  # nosec assert_used
         json.dump(
             {
                 "access_token": self._access_token,
@@ -440,7 +444,15 @@ class DeviceCodeFlowHandler(AuthenticationHandler):
         If no data, or no file is found it will just return without error.
         """
         if token_file.exists():
-            serialized_tokens = json.load(token_file.open())
+            try:
+                with token_file.open() as tf:
+                    serialized_tokens = json.load(tf)
+            except (JSONDecodeError, IOError) as e:
+                logger.warning(
+                    f"Unable to read existing token file ({str(token_file)}),"
+                    f" will require new login: {e}"
+                )
+                return
 
             auth_domain = serialized_tokens["auth_domain"]
             client_id = serialized_tokens["client_id"]
@@ -513,9 +525,9 @@ class DeviceCodeFlowHandler(AuthenticationHandler):
 
         # This method should only be called after a call to _fetch_device_code
         # so these will be set. Asserts to reassure mypy.
-        assert self.device_code_arrival_time is not None  # nosec[assert_used]
-        assert self.device_code_expires_in is not None  # nosec[assert_used]
-        assert self.token_request_interval is not None  # nosec[assert_used]
+        assert self.device_code_arrival_time is not None  # nosec assert_used
+        assert self.device_code_expires_in is not None  # nosec assert_used
+        assert self.token_request_interval is not None  # nosec assert_used
 
         interval = self.token_request_interval
 
