@@ -19,8 +19,6 @@ from typing import (
     cast,
 )
 
-import matplotlib
-import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
 from scipy.stats import kstest
@@ -429,22 +427,6 @@ class MetricCollection:
             cast(Dict, self.results), orient="index", columns=["value"]
         )
 
-    def view_thresholds(self) -> matplotlib.figure.Figure:
-        """Displays chart of how thresholds affect relevant metrics."""
-        fig, ax = plt.subplots()
-
-        for metric_name, metric_vals in cast(
-            Mapping[str, Iterable[float]], self.threshold_metrics
-        ).items():
-            plt.plot(self.thresholds, metric_vals, label=metric_name)
-        ax.legend()
-        ax.set_title("Metric values at various thresholds")
-        ax.set_ylabel("Metric value")
-        ax.set_xlabel("Threshold")
-        plt.close()
-
-        return fig
-
     @staticmethod
     def _get_metrics(problem: MetricsProblem) -> Dict[str, Metric]:
         """Returns metrics appropriate for the task, otherwise simply returns `metrics`.
