@@ -7,7 +7,6 @@ from __future__ import annotations
 
 from abc import ABC, abstractmethod
 from dataclasses import dataclass, field
-from enum import Enum, auto
 import inspect
 import logging
 import os
@@ -53,6 +52,7 @@ from bitfount.types import (
 from bitfount.utils import _add_this_to_list, delegates, seed_all
 
 if TYPE_CHECKING:
+    from bitfount.federated.helper import TaskContext
     from bitfount.metrics import Metric
 
 
@@ -99,17 +99,6 @@ class LoggerConfig:
     )  # type: ignore[assignment] # Reason: default_factory causes type issues
 
 
-class ModelContext(Enum):
-    """Describes the context (modeller or worker) in which the model is running.
-
-    This is used for models where the model differs depending on if it is on the
-    modeller-side or worker-side of the federated process.
-    """
-
-    MODELLER = auto()
-    WORKER = auto()
-
-
 ModelType = TypeVar("ModelType", bound="_BaseModel")
 _model_structure_registry: Dict[str, Type[_BaseModelStructureMixIn]] = {}
 model_structure_registry: Mapping[
@@ -179,7 +168,7 @@ class _BaseModel(
         **kwargs: Any,
     ):
         self.class_name = f"bitfount.{type(self).__name__}"
-        self._context: Optional[ModelContext] = None
+        self._context: Optional[TaskContext] = None
         self.metrics: Optional[MutableMapping[str, Metric]] = None
         self._model: Optional[ModelType] = None
         self._initialised: bool = False
@@ -239,12 +228,12 @@ class _BaseModel(
 
         # If the model is distributed, then we need to get the table name that
         # the Modeller has specified for this particular Pod.
-        pod_identifier: Optional[str] = None
-        if hasattr(self, "pod_identifier"):
+        datastructure_identifier: Optional[str] = None
+        if hasattr(self, "_datastructure_identifier"):
             # Assuring mypy that the attribute exists
-            assert isinstance(self, DistributedModelProtocol)  # nosec[assert_used]
-            pod_identifier = self.pod_identifier
-        table_name = self.datastructure.get_table_name(pod_identifier)
+            assert isinstance(self, DistributedModelProtocol)  # nosec assert_used
+            datastructure_identifier = self._datastructure_identifier
+        table_name = self.datastructure.get_table_name(datastructure_identifier)
 
         # Add the datasource features to the datastructure.
         # This intentionally does not support multi-table datasources because a
@@ -265,7 +254,7 @@ class _BaseModel(
     def initialise_model(
         self,
         data: Optional[BaseSource] = None,
-        context: Optional[ModelContext] = None,
+        context: Optional[TaskContext] = None,
     ) -> None:
         """Can be implemented to initialise model if necessary.
 
@@ -417,7 +406,7 @@ class ClassifierMixIn(_BaseModelRegistryMixIn, _BaseSerializableObjectMixIn):
         if self.datastructure.target is None and hasattr(self, "n_classes"):
             logger.warning(
                 "No target specified in data. Using explicitly provided n_classes."
-                "Note that only inference results will be valid, not training"
+                "Note that only inference results will be valid, not training "
                 "or evaluation for this model and dataset."
             )
         elif self.datastructure.target is not None:
