@@ -8,6 +8,7 @@ from pathlib import Path
 from typing import (
     TYPE_CHECKING,
     Any,
+    Dict,
     Iterable,
     List,
     Optional,
@@ -20,8 +21,12 @@ from typing import (
 import desert
 import yaml
 
-from bitfount.config import BITFOUNT_OUTPUT_DIR
+from bitfount.config import BITFOUNT_DEFAULT_BATCHED_EXECUTION, BITFOUNT_OUTPUT_DIR
 from bitfount.data.datastructure import DataStructure
+from bitfount.federated.algorithms.base import BaseAlgorithmFactory
+from bitfount.federated.algorithms.hugging_face_algorithms.timm_fine_tuning import (
+    TIMMFineTuning,
+)
 from bitfount.federated.authorisation_checkers import IdentityVerificationMethod
 from bitfount.federated.exceptions import AggregatorError
 from bitfount.federated.helper import (
@@ -37,8 +42,14 @@ from bitfount.federated.types import AlgorithmType, ProtocolType
 from bitfount.federated.utils import _ALGORITHMS, _MODEL_STRUCTURES, _MODELS, _PROTOCOLS
 from bitfount.hub.helper import _create_bitfounthub, get_pod_schema
 from bitfount.models.base_models import _BaseModel
-from bitfount.runners.config_schemas import AlgorithmConfig, ModellerConfig, TaskConfig
+from bitfount.runners.config_schemas import (
+    AlgorithmConfig,
+    ModelAlgorithmConfig,
+    ModellerConfig,
+    TaskConfig,
+)
 from bitfount.runners.exceptions import PlugInAlgorithmError, PlugInProtocolError
+from bitfount.runners.utils import dataclass_to_kwargs
 from bitfount.types import DistributedModelProtocol
 
 if TYPE_CHECKING:
@@ -51,7 +62,7 @@ logger = logging.getLogger(__name__)
 
 def setup_modeller_from_config_file(
     path_to_config_yaml: Union[str, PathLike],
-) -> Tuple[_Modeller, List[str], Optional[str], bool]:
+) -> Tuple[_Modeller, List[str], Optional[str], bool, bool]:
     """Creates a modeller from a YAML config file.
 
     Args:
@@ -64,7 +75,6 @@ def setup_modeller_from_config_file(
 
     with open(path_to_config_yaml) as f:
         config_yaml = yaml.safe_load(f)
-
     modeller_config_schema = desert.schema(ModellerConfig)
     modeller_config_schema.context["config_path"] = path_to_config_yaml
 
@@ -74,7 +84,7 @@ def setup_modeller_from_config_file(
 
 def setup_modeller_from_config(
     config: ModellerConfig,
-) -> Tuple[_Modeller, List[str], Optional[str], bool]:
+) -> Tuple[_Modeller, List[str], Optional[str], bool, bool]:
     """Creates a modeller from a loaded config mapping.
 
     Args:
@@ -92,7 +102,10 @@ def setup_modeller_from_config(
     bitfount_hub = _create_bitfounthub(
         username=config.modeller.username, url=config.hub.url, secrets=config.secrets
     )
-
+    if config.batched_execution is None:
+        batched_execution = BITFOUNT_DEFAULT_BATCHED_EXECUTION
+    else:
+        batched_execution = config.batched_execution
     # We assume that if the user has not included a username in
     # a pod identifier that it is their own pod
     pod_identifiers: List[str] = _check_and_update_pod_ids(
@@ -107,9 +120,16 @@ def setup_modeller_from_config(
         identity_verification_method=config.modeller.identity_verification_method,
         private_key_file=config.modeller.private_key_file,
         idp_url=config.modeller._identity_provider_url,
+        project_id=config.project_id,
     )
 
-    return modeller, pod_identifiers, config.project_id, config.run_on_new_data_only
+    return (
+        modeller,
+        pod_identifiers,
+        config.project_id,
+        config.run_on_new_data_only,
+        batched_execution,
+    )
 
 
 def setup_modeller(
@@ -122,6 +142,7 @@ def setup_modeller(
     ] = IdentityVerificationMethod.DEFAULT,
     private_key_file: Optional[Path] = None,
     idp_url: Optional[str] = None,
+    project_id: Optional[str] = None,
 ) -> _Modeller:
     """Creates a modeller.
 
@@ -144,7 +165,7 @@ def setup_modeller(
 
     # Check that the schemas of the given pods match
     # TODO: [BIT-1098] Manage pods with different schemas
-    schema = get_pod_schema(pod_identifiers[0], hub=bitfount_hub)
+    schema = get_pod_schema(pod_identifiers[0], hub=bitfount_hub, project_id=project_id)
     for pod_id in pod_identifiers[1:]:
         aux_schema = get_pod_schema(pod_id, hub=bitfount_hub)
         # We need to check that the schemas have the same contents
@@ -152,30 +173,38 @@ def setup_modeller(
             raise ValueError(
                 "Pod schemas must match in order to be able to train on them."
             )
-
     # Load algorithm from components
     if not isinstance(task_algorithm := task_details.algorithm, list):
         task_algorithm = [cast(AlgorithmConfig, task_details.algorithm)]
 
     algorithm = []
     models = []
+
+    # Create data structure
+    data_config = task_details.data_structure
+    if data_config:
+        if (
+            not data_config.select.include
+            and not data_config.select.exclude
+            and not data_config.table_config.query
+        ):
+            data_config.select.include = schema.tables[0].get_feature_names()
+        data_structure = DataStructure.create_datastructure(
+            table_config=data_config.table_config,
+            select=data_config.select,
+            transform=data_config.transform,
+            assign=data_config.assign,
+            data_split=data_config.data_split,
+        )
+
     for algo in task_algorithm:
         model: Optional[Union[_BaseModel, BitfountModelReference]] = None
-        if algo.model:
-            if not task_details.data_structure:
+
+        if issubclass(type(algo), ModelAlgorithmConfig) and algo.model:
+            if not data_config:
                 raise ValueError(
                     "If a model is provided, a data structure must be provided too."
                 )
-            # Create data structure
-            data_config = task_details.data_structure
-            if not data_config.select.include and not data_config.select.exclude:
-                data_config.select.include = schema.tables[0].get_feature_names()
-            data_structure = DataStructure.create_datastructure(
-                table_config=data_config.table_config,
-                select=data_config.select,
-                transform=data_config.transform,
-                assign=data_config.assign,
-            )
 
             # Create model
             model_details = algo.model
@@ -189,11 +218,11 @@ def setup_modeller(
                 model_name = model_details.name
                 try:
                     model_class = _MODELS[model_name]
-                except KeyError:
+                except KeyError as e:
                     raise KeyError(
                         f"Unable to load built-in model {model_name}; "
                         f"does this pod have the appropriate backend installed?"
-                    )
+                    ) from e
                 # Check if Differential Privacy can be used with this model
                 # The cast here is needed to assuage mypy due to the nested Schema
                 # classes being different; any classes that inherit both _BaseModel
@@ -260,7 +289,7 @@ def setup_modeller(
                     hyperparameters=model_details.hyperparameters,
                     hub=bitfount_hub,
                 )
-                model.get_model()
+                model.get_model(project_id)
                 # We call get_model here to upload it to the hub earlier in the code,
                 # to help us mitigate a race condition between the pod and the modeller,
                 # where the pod is trying to get the model from the hub when it has
@@ -272,31 +301,62 @@ def setup_modeller(
                 )
             models.append(model)
 
+        # Determine algorithm class
+        algorithm_cls: Type[BaseAlgorithmFactory]
         try:
-            algorithm.append(
-                _ALGORITHMS[AlgorithmType(algo.name).name](
-                    **algo.arguments, model=model, pretrained_file=algo.pretrained_file
-                )
-            )
+            # First we see if it is a built-in algorithm class
+            algorithm_cls = _ALGORITHMS[AlgorithmType(algo.name).name]
         except ValueError:
-            # Check if the algorithm is a plugin
+            # If algo.name is not in AlgorithmType then we see if it is a plugin
+            logger.debug(
+                f"Could not find {algo.name} in built-in algorithm classes."
+                f" Trying to load as plugin..."
+            )
             try:
                 algorithm_cls = _ALGORITHMS[algo.name]
-                if model:
-                    algorithm.append(
-                        algorithm_cls(
-                            **algo.arguments,
-                            model=model,
-                            pretrained_file=algo.pretrained_file,
-                        )
-                    )
-                else:
-                    algorithm.append(algorithm_cls(**algo.arguments))
-            # Raise custom error if algorithm not found.
-            except KeyError:
+            except KeyError as e:
                 raise PlugInAlgorithmError(
-                    "The specified plugin algorithm was not found."
-                )
+                    "The specified algorithm was not found as a plugin"
+                    " and is not a built-in algorithm."
+                ) from e
+
+        # Construct algorithm kwargs as needed
+        additional_algo_kwargs: Dict[str, Any] = dict()
+        if model:
+            # If we are working with a model then we must be working with
+            # a model algorithm so can treat it as such
+            additional_algo_kwargs.update(
+                {
+                    "model": model,
+                    "pretrained_file": algo.pretrained_file,
+                    "project_id": project_id,
+                }
+            )
+        # Schema and datastructure already exist separately so they are not part of the
+        # algorithm arguments config and must be added separately
+        elif algorithm_cls == TIMMFineTuning:
+            additional_algo_kwargs.update(schema=schema, datastructure=data_structure)
+
+        # Build and append algorithm instance
+        algo_kwargs = {}
+        if isinstance(algo.arguments, dict):
+            algo_kwargs.update(algo.arguments)
+        elif algo.arguments is not None:
+            # If `.arguments` is not a dict it's likely a nested dataclass all
+            # of its own
+            algo_kwargs.update(dataclass_to_kwargs(algo.arguments))
+
+        algorithm.append(algorithm_cls(**algo_kwargs, **additional_algo_kwargs))
+
+    # Set protocol kwargs
+    protocol_kwargs = {}
+    if isinstance(task_details.protocol.arguments, dict):
+        protocol_kwargs.update(task_details.protocol.arguments)
+    else:
+        # If `.arguments` is not a dict it's likely a nested dataclass all
+        # of its own
+        protocol_kwargs.update(dataclass_to_kwargs(task_details.protocol.arguments))
+
     # Set aggregation options
     if task_details.aggregator is not None:
         if len(models) > 0:
@@ -324,24 +384,26 @@ def setup_modeller(
             secure_aggregation=task_details.aggregator.secure,
             weights=task_details.aggregator.weights,
         )
-        task_details.protocol.arguments["aggregator"] = aggregator
+        protocol_kwargs.update({"aggregator": aggregator})
 
     # Load protocol from components
     try:
         protocol = _PROTOCOLS[ProtocolType(task_details.protocol.name).name](
             algorithm=algorithm if len(algorithm) > 1 else algorithm[0],
-            **task_details.protocol.arguments,
+            **protocol_kwargs,
         )
     except ValueError:
         # Check if the protocol is a plugin
         try:
             protocol = _PROTOCOLS[task_details.protocol.name](
                 algorithm=algorithm if len(algorithm) > 1 else algorithm[0],
-                **task_details.protocol.arguments,
+                **protocol_kwargs,
             )
         # Raise custom error if protocol not found.
-        except KeyError:
-            raise PlugInProtocolError("The specified plugin protocol was not found.")
+        except KeyError as e:
+            raise PlugInProtocolError(
+                "The specified plugin protocol was not found."
+            ) from e
 
     # Create Modeller
     message_service = _create_message_service(
@@ -367,6 +429,7 @@ async def run_modeller_async(
     model_out: Optional[Path] = DEFAULT_MODEL_OUT,
     project_id: Optional[str] = None,
     run_on_new_data_only: bool = False,
+    batched_execution: Optional[bool] = None,
 ) -> Optional[Any]:
     """Runs the modeller.
 
@@ -380,18 +443,22 @@ async def run_modeller_async(
         model_out: The path to save the model out to. Defaults to "./output-model.pt".
         project_id: Project Id the task belongs to.
         run_on_new_data_only: Whether to run the task on new datapoints only.
-                Defaults to False.
+            Defaults to False.
+        batched_execution: Whether to run the task in batched mode. Defaults to False.
 
     Raises:
         PodResponseError: If require_all_pods is true and at least one pod
             identifier specified rejects or fails to respond to a task request.
     """
     # Start task running
+    if batched_execution is None:
+        batched_execution = BITFOUNT_DEFAULT_BATCHED_EXECUTION
     result = await modeller.run_async(
         pod_identifiers,
         require_all_pods=require_all_pods,
         project_id=project_id,
         run_on_new_data_only=run_on_new_data_only,
+        batched_execution=batched_execution,
     )
 
     if model_out:
@@ -411,6 +478,7 @@ def run_modeller(
     model_out: Optional[Path] = DEFAULT_MODEL_OUT,
     project_id: Optional[str] = None,
     run_on_new_data_only: bool = False,
+    batched_execution: Optional[bool] = None,
 ) -> Optional[Any]:
     """Runs the modeller.
 
@@ -424,13 +492,16 @@ def run_modeller(
         model_out: The path to save the model out to. Defaults to "./output-model.pt".
         project_id: Project Id the task belongs to. Defaults to None.
         run_on_new_data_only: Whether to run the task on new datapoints only.
-                Defaults to False.
+            Defaults to False.
+        batched_execution: Whether to run the task in batched mode. Defaults to False.
 
     Raises:
         PodResponseError: If require_all_pods is true and at least one pod
             identifier specified rejects or fails to respond to a task request.
     """
     pod_identifiers = _check_and_update_pod_ids(pod_identifiers, modeller._hub)
+    if batched_execution is None:
+        batched_execution = BITFOUNT_DEFAULT_BATCHED_EXECUTION
     return asyncio.run(
         run_modeller_async(
             modeller,
@@ -439,5 +510,6 @@ def run_modeller(
             model_out,
             project_id,
             run_on_new_data_only,
+            batched_execution,
         )
     )
