@@ -259,7 +259,9 @@ def _get_module_from_file(path_to_module: Path) -> ModuleType:
         sys.modules[module_name] = module
         spec.loader.exec_module(module)  # type: ignore[union-attr] # Reason: See comment # noqa: B950
     except Exception as ex:
-        raise ImportError(f"Unable to load code from {path_to_module}") from ex
+        msg = f"Unable to load code from {path_to_module}"
+        _logger.debug(msg, exc_info=True)
+        raise ImportError(msg) from ex
 
     return module
 
@@ -307,9 +309,9 @@ def _import_module_from_file(
             # If we added the module to sys.modules, remove it as the import failed
             sys.modules.pop(fqual_module_name, None)  # "None" default prevents KeyError
 
-            raise ImportError(
-                f"Unable to load code from {path_to_module} ({str(ex)})"
-            ) from ex
+            msg = f"Unable to load code from {path_to_module}"
+            _logger.debug(msg, exc_info=True)
+            raise ImportError(msg) from ex
 
         return module, module_name
     else:
@@ -535,24 +537,24 @@ def _handle_fatal_error(
 
     How the error is handled will depend on the configuration:
         - Normal, API-based use:
+            - Exception is raised and system errors out.
+            - Full stack trace logged at CRITICAL.
+
+        - Logs limited, API-based use:
             - Exception is raised and system errors out.
             - Limited stack trace/error logged at CRITICAL.
             - Full stack trace logged at DEBUG.
 
-        - Dev, API-based use:
-            - Exception is raised and system errors out.
-            - Full stack trace logged at CRITICAL.
-
         - Normal, CLI-based use:
+            - Exception is captured and sys.exit() is called.
+            - Full stack trace logged at CRITICAL.
+
+        - Logs limited, CLI-based use:
             - Exception is captured and sys.exit() is called.
             - Limited stack trace/error logged at CRITICAL.
             - Full stack trace logged at DEBUG.
 
-        - Dev, CLI-based use:
-            - Exception is captured and sys.exit() is called.
-            - Full stack trace logged at CRITICAL.
-
-    Dev mode functionality can be configured by setting the BITFOUNT_DEV_MODE
+    Logs limited functionality can be configured by setting the BITFOUNT_LIMIT_LOGS
     environment variable to True.
     CLI mode functionality can be configured by setting the BITFOUNT_CLI_MODE
     environment variable to True. This is automatically done if the Bitfount library
@@ -570,7 +572,7 @@ def _handle_fatal_error(
         log_msg = str(ex)
 
     # Determine how full stack trace will be logged
-    if not config.BITFOUNT_DEV_MODE:
+    if config._BITFOUNT_LIMIT_LOGS:
         # Will want to say "see debug log for more details"
         additional_log_msg = ""
         if debug_log_files := _find_logger_filenames(logger, "DEBUG"):
@@ -587,9 +589,10 @@ def _handle_fatal_error(
 
     if config._BITFOUNT_CLI_MODE:
         # Exit instead of raising exception
-        sys.exit(ex)
+        sys.exit(ex)  # type: ignore[arg-type] # Reason: This is allowed
     else:
         # Exceptions get raised
+        # Need to use `raise X` form here as we're not directly in an `except` clause
         raise ex
 
 
@@ -674,18 +677,25 @@ class ExampleSegmentationData:
     ) -> Tuple[int, int, int]:
         """Gets a random location for placing shapes."""
         x = int(
-            width * random.uniform(0.1, 0.9)  # nosec [B311: random] # reason: see below
+            width
+            * random.uniform(
+                0.1, 0.9
+            )  # nosec B311 # "random" usage # reason: see below
         )
         y = int(
             height
-            * random.uniform(0.1, 0.9)  # nosec [B311: random] # reason: see below
+            * random.uniform(
+                0.1, 0.9
+            )  # nosec B311 # "random" usage # reason: see below
         )
         size = int(
             min(width, height)
-            * random.uniform(0.06, 0.12)  # nosec [B311: random] # reason: see below
+            * random.uniform(
+                0.06, 0.12
+            )  # nosec B311 # "random" usage # reason: see below
             * zoom
         )
-        # We only use random for generating a images for a demo segmentation
+        # We only use random for generating images for a demo segmentation
         # dataset used in testing an as an example in tutorial 11
         return (x, y, size)
 
@@ -788,7 +798,7 @@ def delegates(
             to_klass_dict = ancestor.__dict__
             # get the abstract methods
             if hasattr(ancestor, "__abstractmethods__"):
-                abstract_methods = ancestor.__abstractmethods__  # type: ignore[attr-defined] # Reason: hasattr check # noqa: B950
+                abstract_methods = ancestor.__abstractmethods__
             else:
                 abstract_methods = []
 
