@@ -9,6 +9,7 @@ from typing import (
     Awaitable,
     Callable,
     Final,
+    Mapping,
     Optional,
     TypeVar,
     Union,
@@ -159,7 +160,7 @@ def _get_sync_wrapper(
                 # If a connection error occurs, we can retry unless
                 # this is our final attempt
                 if final_retry:
-                    raise ex
+                    raise
                 else:
                     # If the exception contains request info, we can use it
                     if req := ex.request:
@@ -243,7 +244,7 @@ def _get_async_wrapper(
                 # If a connection error occurs, we can retry unless
                 # this is our final attempt
                 if final_retry:
-                    raise ex
+                    raise
                 else:
                     # If the exception contains request info, we can use it
                     try:
@@ -333,7 +334,7 @@ class _AsyncClient(httpx.AsyncClient):
         auth: Union[AuthTypes, UseClientDefault, None] = USE_CLIENT_DEFAULT,
         follow_redirects: Union[bool, UseClientDefault] = USE_CLIENT_DEFAULT,
         timeout: Union[TimeoutTypes, UseClientDefault] = USE_CLIENT_DEFAULT,
-        extensions: Optional[dict] = None,
+        extensions: Optional[Mapping[str, Any]] = None,
     ) -> httpx.Response:
         """See httpx.AsyncClient.request() for information."""  # noqa: D402
         return await super().request(
